# -*- coding: utf-8 -*-

from collective.nitf.config import PROJECTNAME
from collective.nitf.content import INITF
from collective.nitf.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue

import unittest2 as unittest


class CatalogTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.catalog = self.portal['portal_catalog']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.folder.invokeFactory('collective.nitf.content', 'n2')
        self.n1 = self.folder['n1']
        self.n2 = self.folder['n2']

    def test_interface_indexed(self):
        result = self.catalog(object_provides=INITF.__identifier__)
        self.assertEqual(2, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())
        self.assertEqual(result[1].getURL(), self.n2.absolute_url())

    def test_subtitle_indexed(self):
        self.n1.subtitle = 'subtitle'
        self.n1.reindexObject()
        result = self.catalog(subtitle='subtitle')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_byline_indexed(self):
        self.n1.byline = u'Héctor Velarde'
        self.n1.reindexObject()
        result = self.catalog(byline='Héctor')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_genre_indexed(self):
        self.n1.genre = 'Current'
        self.n1.reindexObject()
        result = self.catalog(genre='Current')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_section_indexed(self):
        self.n1.section = 'section'
        self.n1.reindexObject()
        result = self.catalog(section='section')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_urgency_indexed(self):
        self.n1.urgency = '5'
        self.n1.reindexObject()
        result = self.catalog(urgency='5')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_location_indexed(self):
        self.n1.location = u'México, DF'
        self.n1.reindexObject()
        result = self.catalog(location='México')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_searchable_text_indexed(self):
        """ SearchableText must contain id, title, subtitle, abstract, author,
        body text and location as plain text.
        """
        self.n1.title = u'title'
        self.n1.subtitle = u'subtitle'
        self.n1.description = u'abstract'
        self.n1.byline = u'Héctor Velarde'
        self.n1.text = RichTextValue(u'body text', 'text/plain', 'text/html')
        self.n1.location = u'México, DF'
        self.n1.reindexObject()
        result = self.catalog(SearchableText='n1')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='title')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='subtitle')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='abstract')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='Héctor')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='body')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

        result = self.catalog(SearchableText='México')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())

    def test_searchable_text_missing_fields(self):
        """ Adding an NITF with its title only should be enough to get the
        SearchableText index indexed.
        """
        self.folder.invokeFactory('collective.nitf.content', 'nitf')
        nitf = self.folder['nitf']
        nitf.title = u'title'
        nitf.reindexObject()
        result = self.catalog(SearchableText='nitf')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())
        result = self.catalog(SearchableText='title')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        # Now let's start adding the remaining fields one by one, first
        # testing that no results are found, and then they are
        result = self.catalog(SearchableText='subtitle')
        self.assertEqual(0, len(result))

        nitf.subtitle = u'subtitle'
        nitf.reindexObject()
        result = self.catalog(SearchableText='subtitle')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='abstract')
        self.assertEqual(0, len(result))

        nitf.description = u'abstract'
        nitf.reindexObject()
        result = self.catalog(SearchableText='abstract')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='author')
        self.assertEqual(0, len(result))

        nitf.byline = u'Héctor Velarde'
        nitf.reindexObject()
        result = self.catalog(SearchableText='Héctor')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='body')
        self.assertEqual(0, len(result))

        nitf.text = RichTextValue('body text', 'text/plain', 'text/html')
        nitf.reindexObject()
        result = self.catalog(SearchableText='body')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='México')
        self.assertEqual(0, len(result))

        nitf.location = u'México, DF'
        nitf.reindexObject()
        result = self.catalog(SearchableText='México')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        # Finally, let's do some searches to make sure nothing was
        # "de-categorized"
        result = self.catalog(SearchableText='nitf')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='title')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='subtitle')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='abstract')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='Héctor')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='body')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

        result = self.catalog(SearchableText='México')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), nitf.absolute_url())

    def test_catalog_not_lost_on_package_reinstall(self):
        """ Catalog information should not be lost on package reinstall.
        https://github.com/collective/collective.nitf/issues/33
        """
        self.n1.byline = u'Héctor Velarde'
        self.n1.reindexObject()
        qi = self.portal['portal_quickinstaller']
        qi.reinstallProducts(products=[PROJECTNAME])
        result = self.catalog(SearchableText='Héctor')
        self.assertEqual(1, len(result))
        self.assertEqual(result[0].getURL(), self.n1.absolute_url())



# -*- coding: utf-8 -*-

from collective.nitf import _
from collective.nitf.controlpanel import INITFSettings
from collective.z3cform.widgets.multicontent_search_widget import MultiContentSearchFieldWidget
from five import grok
from plone.app.textfield import RichText
from plone.app.textfield.interfaces import ITransformer
from plone.dexterity.content import Container
from plone.app.dexterity.behaviors.metadata import IDublinCore
from plone.directives import form
from plone.formwidget.contenttree import ObjPathSourceBinder
from plone.indexer import indexer
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.component import getUtility


class INITF(form.Schema):
    """A news item based on the News Industry Text Format specification.
    """

    # title = schema.TextLine()
    # nitf/head/title and nitf/body/body.head/hedline/hl1

    form.order_after(subtitle='IDublinCore.title')
    subtitle = schema.TextLine(
        # nitf/body/body.head/hedline/hl2
        title=_(u'Subtitle'),
        description=_(u'help_subtitle',
                      default=u'A subordinate headline for the article.'),
        default=u'',
        missing_value=u'',
        required=False,
    )

    # description = schema.Text()
    # nitf/body/body.head/abstract

    byline = schema.TextLine(
        # nitf/body/body.head/byline/person
        title=_(u'Author'),
        default=u'',
        missing_value=u'',
        required=False,
    )

    text = RichText(
        # nitf/body/body.content
        title=_(u'Body text'),
        required=False,
    )

    genre = schema.Choice(
        # nitf/head/tobject/tobject.property/@tobject.property.type
        title=_(u'Genre'),
        description=_(u'help_genre',
                      default=u'Describes the nature, journalistic or '
                              u'intellectual characteristic of a news '
                              u'object, not specifically its content.'),
        source=u'collective.nitf.AvailableGenres',
    )

    section = schema.Choice(
        # nitf/head/pubdata/@position.section
        title=_(u'Section'),
        description=_(u'help_section',
                      default=u'Named section where the article will '
                              u'appear.'),
        vocabulary=u'collective.nitf.AvailableSections',
    )

    urgency = schema.Choice(
        # nitf/head/docdata/urgency/@ed-urg
        title=_(u'Urgency'),
        description=_(u'help_urgency',
                      default=u'News importance.'),
        vocabulary=u'collective.nitf.Urgencies',
    )

    # XXX: this field uses a special widget that access the most recent items
    # of content types defined in the control panel; see browser.py and
    # controlpanel.py for more information
    relatedItems = RelationList(
        title=_(u'label_related_items', default=u'Related Items'),
        default=[],
        missing_value=[],
        value_type=RelationChoice(title=u"Related",
                                  source=ObjPathSourceBinder()),
        required=False,
    )
    form.widget(relatedItems=MultiContentSearchFieldWidget)

    location = schema.TextLine(
        # nitf/body/body.head/dateline/location
        title=_(u'Location'),
        description=_(u'help_location',
                      default=u'Event location. Where an event took '
                              u'place (as opposed to where the story was '
                              u'written).'),
        default=u'',
        missing_value=u'',
        required=False,
    )

    form.fieldset(
        'categorization',
        label=_(u'Categorization'),
        fields=['relatedItems', 'section', 'urgency', 'genre'],
    )


class NITF(Container):
    grok.implements(INITF)

    def is_empty(self):
        """Return True if the container has no files nor links inside.
        """
        content_filter = {'portal_type': ['File', 'Link']}
        return not self.listFolderContents(content_filter)

    # The purpose of these methods is to emulate those on News Item
    def getImage(self):
        """Return the first Image inside the News Article.
        """
        content_filter = {'portal_type': 'Image'}
        images = self.listFolderContents(content_filter)
        return images[0] if len(images) > 0 else None

    image = getImage  # XXX: a hack to support summary_view

    def imageCaption(self):
        image = self.getImage()
        if image is not None:
            return image.Description()

    def tag(self, **kwargs):
        # tag original implementation returns object title in both, alt and
        # title attributes
        image = self.getImage()
        if image is not None:
            scales = image.restrictedTraverse('@@images')
            if 'scale' in kwargs:
                scale_id = kwargs.get('scale')
                del kwargs['scale']
            else:
                scale_id = 'thumb'
            kwargs['alt'] = image.Description()
            kwargs['title'] = image.Title()
            scale = scales.scale(fieldname='image', scale=scale_id)
            return scale.tag(**kwargs)

    def image_thumb(self):
        """Return a thumbnail."""
        image = self.getImage()
        if image is not None:
            view = image.unrestrictedTraverse('@@images')
            # Return the data
            return view.scale(fieldname='image', scale='thumb').data


@form.default_value(field=INITF['genre'])
def genre_default_value(data):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    return settings.default_genre


@form.default_value(field=INITF['section'])
def section_default_value(data):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    return settings.default_section


@form.default_value(field=INITF['urgency'])
def urgency_default_value(data):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(INITFSettings)
    return settings.default_urgency


# TODO: move this to Dexterity's core
@form.default_value(field=IDublinCore['language'])
def language_default_value(data):
    """ Returns portal's default language or None.
    """
    portal_properties = getToolByName(data, "portal_properties", None)
    if portal_properties is not None:
        site_properties = getattr(portal_properties, 'site_properties', None)
        if site_properties is not None:
            if site_properties.hasProperty('default_language'):
                return site_properties.getProperty('default_language')


@indexer(INITF)
def textIndexer(obj):
    """SearchableText contains id, title, subtitle, abstract, author and body
    text as plain text.
    """
    transformer = ITransformer(obj)
    text = obj.text
    if text:
        text = transformer(obj.text, 'text/plain')

    searchable_text = [safe_unicode(entry) for entry in (
        obj.id,
        obj.Title(),
        obj.subtitle,
        obj.Description(),
        obj.byline,
        text,
        obj.location,
    ) if entry]

    return u" ".join(searchable_text)

grok.global_adapter(textIndexer, name='SearchableText')



# -*- coding: utf-8 -*-

from collective.nitf.config import PROJECTNAME
from Products.CMFCore.utils import getToolByName


def uninstall(portal, reinstall=False):
    if not reinstall:
        profile = 'profile-%s:uninstall' % PROJECTNAME
        setup_tool = getToolByName(portal, 'portal_setup')
        setup_tool.runAllImportStepsFromProfile(profile)
        return "Ran all uninstall steps."



# -*- coding: utf-8 -*-
from collective.nitf.testing import INTEGRATION_TESTING
from plone import api

import unittest2 as unittest

try:
    # Plone >= 4.3
    from Products.CMFPlone.interfaces.syndication import IFeed
except ImportError:
    # Plone 4.2
    from collective.syndication.interfaces import IFeed


class AdaptersTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = api.portal.get()
        with api.env.adopt_roles(['Manager']):
            self.folder = api.content.create(
                type='Folder',
                title='test-folder',
                container=self.portal
            )
            self.n1 = api.content.create(
                type='collective.nitf.content',
                title='n1',
                container=self.folder
            )

    def test_byline_feed_adapter(self):
        self.n1.byline = 'The Author'
        adapted_folder = IFeed(self.folder)
        adapted_n1 = adapted_folder.items.next()
        self.assertEqual(adapted_n1.author_name, 'The Author')



# -*- coding: utf-8 -*-

from collective.nitf.testing import FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import unittest2 as unittest

# XXX: we must replace this tests with a Robot Framework ones
optionflags = doctest.SKIP


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(doctest.DocFileSuite('tests/functional.txt',
                                     optionflags=optionflags,
                                     package='collective.nitf'),
                layer=FUNCTIONAL_TESTING),
        layered(doctest.DocFileSuite('tests/collection.txt',
                                     package='collective.nitf'),
                layer=FUNCTIONAL_TESTING)
    ])
    return suite



# -*- coding:utf-8 -*-

from collective.nitf.config import PROJECTNAME
from five import grok
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import INonInstallable

import logging

PROFILE_ID = 'profile-collective.nitf:default'


class HiddenProfiles(grok.GlobalUtility):

    grok.implements(INonInstallable)
    grok.provides(INonInstallable)
    grok.name('collective.nitf')

    def getNonInstallableProfiles(self):
        profiles = [
            'collective.nitf:uninstall',
            'collective.nitf.upgrades.v1007:default',
        ]
        return profiles


class Empty:
    pass


def add_catalog_indexes(context, logger=None):
    """ Method to add our wanted indexes to the portal_catalog. See
    http://maurits.vanrees.org/weblog/archive/2009/12/catalog for more
    information.
    """
    def extras(title, index_type='Okapi BM25 Rank',
               lexicon_id='plone_lexicon'):
        # See http://old.zope.org/Members/dedalu/ZCTextIndex_python
        extras = Empty()
        extras.doc_attr = title
        extras.index_type = index_type
        extras.lexicon_id = lexicon_id
        return extras

    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    profile = 'profile-collective.nitf:default'
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(profile, 'catalog')

    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    wanted = (
        ('subtitle', 'ZCTextIndex'),
        ('byline', 'ZCTextIndex'),
        ('genre', 'FieldIndex'),
        ('section', 'FieldIndex'),
        ('urgency', 'FieldIndex'),
        ('location', 'ZCTextIndex'),
        ('SearchableText', 'ZCTextIndex'),
    )

    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            if meta_type == 'ZCTextIndex':
                catalog.addIndex(name, meta_type, extras(name))
            else:
                catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info("Added %s for field %s.", meta_type, name)

    if len(indexables) > 0:
        logger.info("Indexing new indexes %s.", ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def import_various(context):
    """ Import step for configuration that is not handled in XML files.
    """
    # Only run step if a flag file is present
    if context.readDataFile('collective.nitf.marker.txt') is None:
        return

    logger = context.getLogger(PROJECTNAME)
    site = context.getSite()
    add_catalog_indexes(site, logger)


def add_galleria_js(context, logger=None):
    """
    """
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    profile = 'profile-collective.js.galleria:default'
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(profile)


def remove_collapsible_js(context, logger=None):
    """
    """
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    portal_js = getToolByName(context, 'portal_javascripts')
    portal_css = getToolByName(context, 'portal_css')
    portal_js.manage_removeScript("++resource++collective.nitf/jquery.collapsible-v.2.1.3.js")
    portal_css.manage_removeStylesheet("++resource++collective.nitf/collapsible.css")


def charcount_control_panel_update(context):
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'plone.app.registry')
    context.runImportStepFromProfile(PROFILE_ID, 'jsregistry')
    context.runImportStepFromProfile(PROFILE_ID, 'cssregistry')


def default_values_update(context, logger=None):
    """
    """
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    catalog = getToolByName(context, 'portal_catalog')
    for item in catalog(portal_type='collective.nitf.content'):
        obj = item.getObject()
        if None in (obj.id, obj.Title(), obj.subtitle,
                    obj.Description(), obj.byline, obj.text,
                    obj.location):
            obj.reindexObject(idxs=['SearchableText'])


def upgrade_z3cformwidgets(context, logger=None):
    """
    """
    if logger is None:
        # Called as upgrade step: define our own logger
        logger = logging.getLogger(PROJECTNAME)

    profile = 'profile-collective.z3cform.widgets:upgrade_1_to_2'
    setup = getToolByName(context, 'portal_setup')
    setup.runAllImportStepsFromProfile(profile)



# -*- coding: utf-8 -*-

from collective.nitf.config import DEFAULT_GENRE, DEFAULT_URGENCY
from collective.nitf.config import DEFAULT_RELATABLE_CONTENT_TYPES
from collective.nitf.config import PROJECTNAME
from collective.nitf.controlpanel import INITFSettings
from collective.nitf.testing import INTEGRATION_TESTING
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter
from zope.component import getUtility

import unittest


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.controlpanel = self.portal['portal_controlpanel']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_controlpanel_has_view(self):
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name='nitf-settings')
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized, self.portal.restrictedTraverse, '@@nitf-settings')

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertIn('nitf', actions, 'control panel not installed')

    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']
        qi.uninstallProducts(products=[PROJECTNAME])
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertNotIn('nitf', actions, 'control panel not removed')


class RegistryTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(INITFSettings)

    def test_available_sections_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'available_sections'))
        self.assertEqual(self.settings.available_sections, set([]))

    def test_default_section_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_section'))
        self.assertEqual(self.settings.default_section, None)

    def test_available_genres_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'available_genres'))
        self.assertEqual(self.settings.available_genres, [])

    def test_default_genre_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_genre'))
        self.assertEqual(self.settings.default_genre, DEFAULT_GENRE)

    def test_default_urgency_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'default_urgency'))
        self.assertEqual(self.settings.default_urgency, DEFAULT_URGENCY)

    def test_relatable_content_types_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'relatable_content_types'))
        self.assertEqual(
            self.settings.relatable_content_types,
            DEFAULT_RELATABLE_CONTENT_TYPES,
        )

    def test_records_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        qi.uninstallProducts(products=[PROJECTNAME])

        BASE_REGISTRY = 'collective.nitf.controlpanel.INITFSettings.%s'
        records = [
            BASE_REGISTRY % 'available_sections',
            BASE_REGISTRY % 'default_section',
            BASE_REGISTRY % 'available_genres',
            BASE_REGISTRY % 'default_genre',
            BASE_REGISTRY % 'default_urgency',
            BASE_REGISTRY % 'relatable_content_types',
        ]

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
from collective.nitf import _
from collective.nitf import config
from plone.app.registry.browser import controlpanel
from z3c.form import field
from z3c.form import group
from zope import schema
from zope.component import getUtility
from zope.interface import alsoProvides
from plone.registry.interfaces import IRegistry
from plone.directives import form

PORTALTYPES = 'plone.app.vocabularies.ReallyUserFriendlyTypes'


class INITFSettings(form.Schema):
    """ Interface for the control panel form.
    """
    form.widget(available_sections="z3c.form.browser.textlines.TextLinesFieldWidget")
    available_sections = schema.Set(
        title=_(u'Available Sections'),
        description=_(u"List of available sections in the site."),
        required=True,
        default=set(),
        value_type=schema.TextLine(title=_(u'Section')),
    )

    default_section = schema.Choice(
        title=_(u"Default Section"),
        vocabulary=u'collective.nitf.AvailableSections',
        required=False,
    )

    available_genres = schema.List(
        title=_(u"Available Genres"),
        description=_(u"Select the list of available genres in the site. "
                      u"Genres indicate a nature, journalistic or "
                      u"intellectual characteristic of items."),
        required=False,
        default=[],
        value_type=schema.Choice(vocabulary=u'collective.nitf.Genres'),
    )

    default_genre = schema.Choice(
        title=_(u"Default Genre"),
        vocabulary=u'collective.nitf.Genres',
        required=False,
        default=config.DEFAULT_GENRE,
    )

    default_urgency = schema.Choice(
        title=_(u"Default Urgency"),
        vocabulary=u'collective.nitf.Urgencies',
        required=False,
        default=config.DEFAULT_URGENCY,
    )

    relatable_content_types = schema.List(
        title=_(u"Relatable Content Types"),
        description=_(u"Only objects of these content types will be "
                      u"listed in the related items widget."),
        required=False,
        default=config.DEFAULT_RELATABLE_CONTENT_TYPES,
        # we are going to list only the main content types in the widget
        value_type=schema.Choice(vocabulary=PORTALTYPES),
    )


class INITFCharCountSettings(form.Schema):

    show_title_counter = schema.Bool(
        title=_(u'label_show_title_counter',
                default=u"Show a counter for the title?"),
        description=_(u'help_show_title_counter',
                      default=u"If selected, a character counter will show "
                              u"the lenght of the field."),
        required=False,
        default=False,
    )

    title_max_chars = schema.Int(
        title=_(u'label_title_max_chars',
                default=u"Maximum title length"),
        description=_(u'help_title_max_chars',
                      default=u"The limit is just visual; validation is not enforced."),
        default=100,
        required=False,
    )

    title_optimal_chars = schema.Int(
        title=_(u'label_title_optimal_chars',
                default=u"Optimal title length"),
        description=_(u"help_title_optimal_chars",
                      default=u"The optimal length will be between this "
                              u"number and the maximum length of the field."),
        default=100,
        required=False,
    )

    show_description_counter = schema.Bool(
        title=_(u'label_show_description_counter',
                default=u"Show a character counter in the description?"),
        description=_(u'help_show_description_counter',
                      default=u"If selected, a character counter will show "
                              u"the lenght of the field."),
        required=False,
        default=False,
    )

    description_max_chars = schema.Int(
        title=_(u'label_description_max_chars',
                default=u"Maximum description length"),
        description=_(u'help_description_max_chars',
                      default=u"The limit is just visual; validation is not enforced."),
        default=200,
        required=False,
    )

    description_optimal_chars = schema.Int(
        title=_(u'label_description_optimal_chars',
                default=u"Optimal description length"),
        description=_(u"help_description_optimal_chars",
                      default=u"The optimal length will be between this "
                              u"number and the maximum length of the field."),
        default=200,
        required=False,
    )


class NITFGroup(group.Group):
    label = _(u"Default")
    description = _("Default Configuration")
    fields = field.Fields(INITFSettings)


class NITFChartCountGroup(group.Group):
    label = _(u"Character Counter")
    description = _("Character Counter Configuration")
    fields = field.Fields(INITFCharCountSettings)


class INITFSchema(INITFSettings, INITFCharCountSettings):
    """
    """


class NITFSettingsEditForm(controlpanel.RegistryEditForm):
    schema = INITFSchema
    label = _(u"NITF Settings")
    description = _(u"Here you can modify the settings for collective.nitf.")

    fields = INITFSettings
    groups = (NITFChartCountGroup,)

    def getContent(self):
        return AbstractRecordsProxy(self.schema)


class NITFSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = NITFSettingsEditForm


class AbstractRecordsProxy(object):
    """Multiple registry schema proxy.

    This class supports schemas that contain derived fields. The
    settings will be stored with respect to the individual field
    interfaces.
    """

    def __init__(self, schema):
        state = self.__dict__
        state["__registry__"] = getUtility(IRegistry)
        state["__proxies__"] = {}
        state["__schema__"] = schema
        alsoProvides(self, schema)

    def __getattr__(self, name):
        try:
            field = self.__schema__[name]
        except KeyError:
            raise AttributeError(name)
        else:
            proxy = self._get_proxy(field.interface)
            return getattr(proxy, name)

    def __setattr__(self, name, value):
        try:
            field = self.__schema__[name]
        except KeyError:
            self.__dict__[name] = value
        else:
            proxy = self._get_proxy(field.interface)
            return setattr(proxy, name, value)

    def __repr__(self):
        return "<AbstractRecordsProxy for %s>" % self.__schema__.__identifier__

    def _get_proxy(self, interface):
        proxies = self.__proxies__
        return proxies.get(interface) or proxies.setdefault(interface,
                                                            self.__registry__.forInterface(interface))



# -*- coding: utf-8 -*-
from collective.nitf.content import INITF
from zope.component import adapts

try:
    # Plone >= 4.3
    from Products.CMFPlone.browser.syndication.adapters import DexterityItem
    from Products.CMFPlone.interfaces.syndication import IFeed
except ImportError:
    # Plone 4.2
    from collective.syndication.adapters import DexterityItem
    from collective.syndication.interfaces import IFeed


class BylineFeed(DexterityItem):

    """Adapter to honor the author of a News Article in the syndication feed."""

    adapts(INITF, IFeed)

    @property
    def author_name(self):
        return self.context.byline



# -*- coding: utf-8 -*-

from collective.nitf import config as c
from collective.nitf.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest2 as unittest


class CollectionTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.folder.invokeFactory('Collection', 'c1')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.c1 = self.folder['c1']
        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            }
        ])

    def test_urgency_filter(self, ):
        self.assertEqual(len(self.c1.queryCatalog()), 0)

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']
        self.n1.urgency = c.LOW
        self.n1.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 1)

        self.folder.invokeFactory('collective.nitf.content', 'n2')
        self.n2 = self.folder['n2']
        self.n2.urgency = c.NORMAL
        self.n2.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 2)

        self.folder.invokeFactory('collective.nitf.content', 'n3')
        self.n3 = self.folder['n3']
        self.n3.urgency = c.HIGH
        self.n3.reindexObject()
        self.assertEqual(len(self.c1.queryCatalog()), 3)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.LOW]
            }
        ])
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(self.n1, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.NORMAL]
            }
        ])
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(self.n2, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.HIGH]
            }
        ])
        expected = self.c1.queryCatalog()[0].getObject()
        self.assertEqual(self.n3, expected)

        self.c1.setQuery([
            {
                'i': 'portal_type',
                'o': 'plone.app.querystring.operation.selection.is',
                'v': ['collective.nitf.content']
            },
            {
                'i': 'urgency',
                'o': 'plone.app.querystring.operation.intselection.is',
                'v': [c.NORMAL, c.HIGH]
            }
        ])
        expected = [b.getObject() for b in self.c1.queryCatalog()]
        self.assertEqual([self.n2, self.n3], expected)



# -*- coding: utf-8 -*-
"""Setup for tests in the package.

We need to install collective.syndication to test the BylineFeed
adapter under Plone 4.2.
"""
import os
import random
import string
from StringIO import StringIO
from PIL import Image

from collective.nitf.controlpanel import INITFSettings
from plone import api
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

PLONE_VERSION = api.env.plone_version()


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        if PLONE_VERSION.startswith('4.2'):
            import collective.syndication
            self.loadZCML(package=collective.syndication)

        import collective.nitf
        self.loadZCML(package=collective.nitf)

    def setUpPloneSite(self, portal):
        if PLONE_VERSION.startswith('4.2'):
            self.applyProfile(portal, 'collective.syndication:default')

        self.applyProfile(portal, 'collective.nitf:default')
        portal_workflow = portal['portal_workflow']
        portal_workflow.setChainForPortalTypes(
            ('collective.nitf.content',), 'simple_publication_workflow')


def generate_jpeg(width, height):
    # Mandelbrot fractal
    # FB - 201003254
    # drawing area
    xa = -2.0
    xb = 1.0
    ya = -1.5
    yb = 1.5
    maxIt = 25  # max iterations allowed
    # image size
    image = Image.new("RGB", (width, height))
    c = complex(random.random() * 2.0 - 1.0, random.random() - 0.5)

    for y in range(height):
        zy = y * (yb - ya) / (height - 1) + ya
        for x in range(width):
            zx = x * (xb - xa) / (width - 1) + xa
            z = complex(zx, zy)
            for i in range(maxIt):
                if abs(z) > 2.0:
                    break
                z = z * z + c
            r = i % 4 * 64
            g = i % 8 * 32
            b = i % 16 * 16
            image.putpixel((x, y), b * 65536 + g * 256 + r)

    output = StringIO()
    image.save(output, format="PNG")
    return output.getvalue()


def generate_text(size):
    chars = string.letters + string.digits
    return ''.join(random.choice(chars) for x in range(size))


# TODO: simplify this using a testfixture profile
class RobotFixture(Fixture):

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        self.applyProfile(portal, 'collective.nitf:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        portal.invokeFactory('collective.nitf.content', 'n1')
        portal.n1.invokeFactory('Image', 'img1')
        portal.n1.invokeFactory('Image', 'img2')
        portal.n1.invokeFactory('Image', 'img3')
        portal.n1.img1.setImage(generate_jpeg(50, 50))
        portal.n1.img2.setImage(generate_jpeg(50, 50))
        portal.n1.img3.setImage(generate_jpeg(50, 50))
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFSettings)
        # FIXME: this needs to be available by default
        settings.available_sections = set([u'Tommy'])
        settings.available_genres = [u'Current']
        open('/tmp/img1.jpg', 'w').write(generate_jpeg(50, 50))
        open('/tmp/txt1.txt', 'w').write(generate_text(256))

    def tearDownPloneSite(self, portal):
        os.remove('/tmp/img1.jpg')
        os.remove('/tmp/txt1.txt')


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='collective.nitf:Integration',
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='collective.nitf:Functional',
)

ROBOT_FIXTURE = RobotFixture()
ROBOT_TESTING = FunctionalTesting(
    bases=(ROBOT_FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name="collective.nitf:Robot")



# -*- coding: utf-8 -*-

from collective.nitf.testing import FUNCTIONAL_TESTING
from plone.testing.z2 import Browser

import pkg_resources
import unittest

PLONE_VERSION = pkg_resources.require("Plone")[0].version


class Plone43TestCase(unittest.TestCase):

    layer = FUNCTIONAL_TESTING

    @unittest.skipUnless('4.3' in PLONE_VERSION, "Plone 4.3 specific")
    def test_static_resource_grokker(self):
        """Grok does not register automatically the static resources anymore
        see: http://svn.zope.org/five.grok/trunk/src/five/grok/meta.py?rev=123298&r1=112163&r2=123298
        """
        portal = self.layer['portal']
        app = self.layer['app']

        browser = Browser(app)
        portal_url = portal.absolute_url()

        browser.open('%s/++resource++collective.nitf' % portal_url)
        self.assertEqual(browser.headers['status'], '200 Ok')



# -*- coding: utf-8 -*-
from collective.nitf.controlpanel import INITFSettings
from collective.nitf.portlets import latest_sectionable_nitf
from collective.nitf.testing import INTEGRATION_TESTING
from DateTime import DateTime
from plone.app.portlets.storage import PortletAssignmentMapping
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.portlets.interfaces import IPortletAssignment
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRenderer
from plone.portlets.interfaces import IPortletType
from plone.registry.interfaces import IRegistry
from zope.component import getUtility, getMultiAdapter

import unittest2 as unittest

PLONE42 = True
try:
    import plone.app.collection
    assert plone.app.collection  # Pyflakes
except ImportError:
    PLONE42 = False


class PortletTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_portlet_type_registered(self):
        name = 'collective.nitf.LatestSectionableNITFPortlet'
        latest_sectionable = getUtility(IPortletType, name=name)
        self.assertEqual(latest_sectionable.addview, name)

    def test_interfaces(self):
        last_sectionable = latest_sectionable_nitf.Assignment()
        self.assertTrue(IPortletAssignment.providedBy(last_sectionable))
        self.assertTrue(IPortletDataProvider.providedBy(last_sectionable.data))

    def test_invoke_add_view(self):
        name = 'collective.nitf.LatestSectionableNITFPortlet'
        latest_sectionable = getUtility(IPortletType, name=name)
        path = '++contextportlets++plone.leftcolumn'
        mapping = self.portal.restrictedTraverse(path)

        for m in mapping.keys():
            del mapping[m]

        addview = mapping.restrictedTraverse('+/' + latest_sectionable.addview)
        addview.createAndAdd(data={})

        self.assertEqual(len(mapping), 1)
        self.assertTrue(isinstance(mapping.values()[0],
                                   latest_sectionable_nitf.Assignment))

    def test_invoke_edit_view(self):
        mapping = PortletAssignmentMapping()
        request = self.request

        mapping['latest_nitf'] = latest_sectionable_nitf.Assignment()

        editview = getMultiAdapter((mapping['latest_nitf'], request),
                                   name='edit')
        self.assertTrue(isinstance(editview, latest_sectionable_nitf.EditForm))

    def test_obtain_renderer(self):
        context = self.portal
        request = self.request
        view = context.restrictedTraverse('@@plone')
        manager = getUtility(IPortletManager, name='plone.rightcolumn',
                             context=self.portal)

        assgmnt1 = latest_sectionable_nitf.Assignment()

        renderer1 = getMultiAdapter(
            (context, request, view, manager, assgmnt1), IPortletRenderer)

        self.assertTrue(isinstance(renderer1,
                                   latest_sectionable_nitf.Renderer))


class RenderTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def set_default_workflow(self):
        # setup default workflow in tests
        types = ('collective.nitf.content',)
        self.wf.setChainForPortalTypes(types, 'simple_publication_workflow')

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.catalog = self.portal.portal_catalog
        self.wf = self.portal.portal_workflow

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        self.folder = self.portal['test-folder']

        self.set_default_workflow()
        # Let's create 3 sections in the registry
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFSettings)
        settings.available_sections = set([u"Section 1",
                                           u"Section 2",
                                           u"Section 3"])

        # Let's create 15 nitf's for each of 3 different sections
        for index in range(1, 16):
            self.folder.invokeFactory('collective.nitf.content',
                                      'section1-nitf-%s' % index)
            n = self.folder['section1-nitf-%s' % index]
            n.title = "Section 1 Nitf %s" % index
            n.section = "Section 1"
            n.genre = "Genre %s" % index
            n.created = DateTime("%(year)s/1/%(index)s %(index)s:00:00" %
                                 {'year': DateTime().year(),
                                  'index': index})
            n.reindexObject()
            # After 5 indexes, publish
            if index % 5 == 0:
                self.wf.doActionFor(n, "publish")

            self.folder.invokeFactory('collective.nitf.content',
                                      'section2-nitf-%s' % index)
            n = self.folder['section2-nitf-%s' % index]
            n.title = "Section 2 Nitf %s" % index
            n.section = "Section 2"
            n.genre = "Genre %s" % index
            n.created = DateTime("%(year)s/2/%(index)s %(index)s:00:00" %
                                 {'year': DateTime().year(),
                                  'index': index})
            n.reindexObject()
            # After 5 indexes, publish
            if index % 5 == 0:
                self.wf.doActionFor(n, "publish")

            self.folder.invokeFactory('collective.nitf.content',
                                      'section3-nitf-%s' % index)
            n = self.folder['section3-nitf-%s' % index]
            n.title = "Section 3 Nitf %s" % index
            n.section = "Section 3"
            n.genre = "Genre %s" % index
            n.created = DateTime("%(year)s/3/%(index)s %(index)s:00:00" %
                                 {'year': DateTime().year(),
                                  'index': index})

            n.reindexObject()
            # After 5 indexes, publish
            if index % 5 == 0:
                self.wf.doActionFor(n, "publish")

        self.default_query = {'Type': ('News Article',),
                              'sort_on': 'created',
                              'sort_order': 'reverse',
                              'sort_limit': 10}

    def renderer(self, context=None, request=None, view=None, manager=None,
                 assignment=None):
        context = context or self.portal
        request = request or self.request
        view = view or self.portal.restrictedTraverse('@@plone')
        manager = manager or getUtility(
            IPortletManager, name='plone.rightcolumn', context=self.portal)

        return getMultiAdapter((context, request, view, manager, assignment),
                               IPortletRenderer)

    def test_render(self):

        assgmnt1 = latest_sectionable_nitf.Assignment()

        r1 = self.renderer(context=self.portal,
                           assignment=assgmnt1)

        r1 = r1.__of__(self.portal)
        r1.update()

    def test_default_search_for_nitf(self):
        assgmnt1 = latest_sectionable_nitf.Assignment()

        r1 = self.renderer(context=self.portal,
                           assignment=assgmnt1)

        r1 = r1.__of__(self.portal)

        view = getMultiAdapter((r1, self.request), name="nitf-filter-list")
        results = view.getResults()

        self.assertEqual(len(results), 10)
        query = self.default_query

        catalog_results = self.catalog(**query)[:10]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        results = view.getResults(limit=45)

        self.assertEqual(len(results), 45)

        query = self.default_query
        query['sort_limit'] = 45

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

    def test_filter_sections(self):
        assgmnt1 = latest_sectionable_nitf.Assignment()

        r1 = self.renderer(context=self.portal,
                           assignment=assgmnt1)

        r1 = r1.__of__(self.portal)

        view = getMultiAdapter((r1, self.request), name="nitf-filter-list")

        for section in [("section 1", "Section 1"),
                        ("section 2", "Section 2"),
                        ("section 3", "Section 3")]:
            results = view.getResults(section=section[0], limit=45)

            self.assertEqual(len(results), 15)
            query = self.default_query
            query['sort_limit'] = 45
            query['section'] = section[1]

            catalog_results = self.catalog(**query)[:45]

            self.assertEqual([i.id for i in results],
                             [i.id for i in catalog_results])

    @unittest.skipIf(PLONE42, "Topics are disabled by default on Plone 4.2")
    def test_modifying_query_through_collection(self):

        assgmnt1 = latest_sectionable_nitf.Assignment()

        r1 = self.renderer(context=self.portal,
                           assignment=assgmnt1)

        r1 = r1.__of__(self.portal)

        view = getMultiAdapter((r1, self.request), name="nitf-filter-list")

        self.folder.invokeFactory('Topic', id='collection')
        topic = self.folder['collection']
        uid = topic.UID()
        # Choose a different type to show. This key should be ignored
        type_crit = topic.addCriterion('Type', 'ATPortalTypeCriterion')
        type_crit.setValue('Document')
        topic.reindexObject()

        results = view.getResults(limit=45, uid=uid)

        self.assertEqual(len(results), 45)
        query = self.default_query
        query['sort_limit'] = 45

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        # Now, let's add a section, it should also be ignored
        section_crit = topic.addCriterion('section', 'ATSimpleStringCriterion')
        section_crit.setValue('Section 1')
        topic.reindexObject()

        results = view.getResults(limit=45, uid=uid)

        self.assertEqual(len(results), 45)
        query = self.default_query

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        # Now, let's just filter published content, we only get 9 results
        state_crit = topic.addCriterion('review_state',
                                        'ATSimpleStringCriterion')
        state_crit.setValue('published')
        topic.reindexObject()

        results = view.getResults(limit=45, uid=uid)

        self.assertEqual(len(results), 9)
        query = self.default_query
        query['review_state'] = 'published'

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        # Now, let's sort results by section and in reverse
        topic.addCriterion('section', 'ATSortCriterion')
        topic.setSortCriterion('section', True)
        topic.reindexObject()

        results = view.getResults(limit=45, uid=uid)

        self.assertEqual(len(results), 9)
        query = self.default_query
        query['sort_on'] = 'section'

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        # Let's now sort it by section but not in reverse
        topic.setSortCriterion('section', False)
        topic.reindexObject()

        results = view.getResults(limit=45, uid=uid)

        self.assertEqual(len(results), 9)
        query = self.default_query
        del query['sort_order']

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])

        # Finally, let's just get results filtered for a particular section

        results = view.getResults(section="section 1", limit=45, uid=uid)

        self.assertEqual(len(results), 3)
        query = self.default_query
        query['section'] = "Section 1"

        catalog_results = self.catalog(**query)[:45]

        self.assertEqual([i.id for i in results],
                         [i.id for i in catalog_results])



# -*- coding: utf-8 -*-

from collective.nitf.content import INITF
from collective.nitf.testing import INTEGRATION_TESTING
from plone.app.referenceablebehavior.referenceable import IReferenceable
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.fti import DexterityFTI
from plone.dexterity.interfaces import IDexterityFTI
from plone.uuid.interfaces import IAttributeUUID
from StringIO import StringIO
from zope.component import createObject
from zope.component import queryUtility

import unittest2 as unittest

zptlogo = (
    'GIF89a\x10\x00\x10\x00\xd5\x00\x00\xff\xff\xff\xff\xff\xfe\xfc\xfd\xfd'
    '\xfa\xfb\xfc\xf7\xf9\xfa\xf5\xf8\xf9\xf3\xf6\xf8\xf2\xf5\xf7\xf0\xf4\xf6'
    '\xeb\xf1\xf3\xe5\xed\xef\xde\xe8\xeb\xdc\xe6\xea\xd9\xe4\xe8\xd7\xe2\xe6'
    '\xd2\xdf\xe3\xd0\xdd\xe3\xcd\xdc\xe1\xcb\xda\xdf\xc9\xd9\xdf\xc8\xd8\xdd'
    '\xc6\xd7\xdc\xc4\xd6\xdc\xc3\xd4\xda\xc2\xd3\xd9\xc1\xd3\xd9\xc0\xd2\xd9'
    '\xbd\xd1\xd8\xbd\xd0\xd7\xbc\xcf\xd7\xbb\xcf\xd6\xbb\xce\xd5\xb9\xcd\xd4'
    '\xb6\xcc\xd4\xb6\xcb\xd3\xb5\xcb\xd2\xb4\xca\xd1\xb2\xc8\xd0\xb1\xc7\xd0'
    '\xb0\xc7\xcf\xaf\xc6\xce\xae\xc4\xce\xad\xc4\xcd\xab\xc3\xcc\xa9\xc2\xcb'
    '\xa8\xc1\xca\xa6\xc0\xc9\xa4\xbe\xc8\xa2\xbd\xc7\xa0\xbb\xc5\x9e\xba\xc4'
    '\x9b\xbf\xcc\x98\xb6\xc1\x8d\xae\xbaFgs\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    '\x00,\x00\x00\x00\x00\x10\x00\x10\x00\x00\x06z@\x80pH,\x12k\xc8$\xd2f\x04'
    '\xd4\x84\x01\x01\xe1\xf0d\x16\x9f\x80A\x01\x91\xc0ZmL\xb0\xcd\x00V\xd4'
    '\xc4a\x87z\xed\xb0-\x1a\xb3\xb8\x95\xbdf8\x1e\x11\xca,MoC$\x15\x18{'
    '\x006}m\x13\x16\x1a\x1f\x83\x85}6\x17\x1b $\x83\x00\x86\x19\x1d!%)\x8c'
    '\x866#\'+.\x8ca`\x1c`(,/1\x94B5\x19\x1e"&*-024\xacNq\xba\xbb\xb8h\xbeb'
    '\x00A\x00;')


class ContentTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_adding(self):
        self.assertTrue(INITF.providedBy(self.n1))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        self.assertIsNotNone(fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        schema = fti.lookupSchema()
        self.assertEqual(INITF, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(INITF.providedBy(new_object))

    def test_is_referenceable(self):
        self.assertTrue(IReferenceable.providedBy(self.n1))
        self.assertTrue(IAttributeUUID.providedBy(self.n1))

    def test_locking_behavior_available(self):
        from plone.app.lockingbehavior.behaviors import ILocking
        self.assertTrue(ILocking.providedBy(self.n1))

    def test_action_is_registered(self):
        fti = queryUtility(IDexterityFTI, name='collective.nitf.content')
        actions = [a.id for a in fti.listActions()]
        self.assertIn('media', actions)

    def test_is_empty(self):
        # the new article has no content inside
        self.assertTrue(self.n1.is_empty())

        # Image doesn't count, so it should be still empty
        self.n1.invokeFactory('Image', 'foo')
        self.assertTrue(self.n1.is_empty())

        # File and Link count
        self.n1.invokeFactory('File', 'bar')
        self.assertFalse(self.n1.is_empty())
        self.n1.manage_delObjects('bar')

        self.n1.invokeFactory('Link', 'baz')
        self.assertFalse(self.n1.is_empty())

    def test_image_scale(self):
        self.assertIsNone(self.n1.getImage())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        view = self.n1.restrictedTraverse('@@images')
        scale = view.scale('image', 'thumb')
        self.assertEqual(scale.height, 16)
        self.assertEqual(scale.width, 16)

    def test_getImage(self):
        self.assertIsNone(self.n1.getImage())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        image = self.n1.getImage()
        self.assertEqual(image.id, 'foo')
        self.assertEqual(image.Title(), 'bar')
        self.assertEqual(image.Description(), 'baz')

    def test_imageCaption(self):
        imageCaption = self.n1.imageCaption
        self.assertIsNone(imageCaption())
        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        self.assertEqual(imageCaption(), 'baz')

    def test_image_thumb(self):
        thumb = self.n1.image_thumb

        # no images in news article, so no thumb must be present
        self.assertIsNone(thumb())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        self.assertIsNotNone(thumb())

    def test_tag(self):
        tag = self.n1.tag

        # no images in news article, so no tag must be present
        self.assertIsNone(tag())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))

        # image title must be in both, alt and title attributes
        self.assertIn('alt="baz" title="bar"', tag())

        # image size
        self.assertIn('height="16" width="16', tag())

        # image scale using @@images
        self.assertIn('src="http://nohost/plone/test-folder/n1/foo/@@images/',
                      tag(scale='preview'))

        # image class
        self.assertIn('class="myClass"', tag(css_class='myClass'))


IMAGE_SCHEMA = '''<model xmlns="http://namespaces.plone.org/supermodel/schema"
       xmlns:marshal="http://namespaces.plone.org/supermodel/marshal"
       xmlns:i18n="http://xml.zope.org/namespaces/i18n"
       i18n:domain="plone">
  <schema>
    <field name="title" type="zope.schema.TextLine">
      <description />
      <required>False</required>
      <title>Title</title>
    </field>
    <field name="description" type="zope.schema.Text">
      <description />
      <required>False</required>
      <title>Description</title>
    </field>
    <field name="image" type="plone.namedfile.field.NamedBlobImage"
        marshal:primary="true">
      <description />
      <title i18n:translate="Image">Image</title>
    </field>
  </schema>
</model>'''


def dummy_image(data):
    from plone.namedfile.file import NamedBlobImage
    return NamedBlobImage(
        data=data,
        filename=u'zptlogo.gif')


class DexterityImageTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUpImageType(self):
        portal = self.portal
        tt = portal.portal_types
        del tt['Image']
        fti = DexterityFTI('Image')
        portal.portal_types._setObject('Image', fti)
        fti.model_source = IMAGE_SCHEMA

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.setUpImageType()
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_image_scale(self):
        self.assertIsNone(self.n1.getImage())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=dummy_image(zptlogo))
        view = self.n1.restrictedTraverse('@@images')
        scale = view.scale('image', 'thumb')
        self.assertEqual(scale.height, 16)
        self.assertEqual(scale.width, 16)

    def test_getImage(self):
        self.assertIsNone(self.n1.getImage())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=dummy_image(zptlogo))
        image = self.n1.getImage()
        self.assertEqual(image.id, 'foo')
        self.assertEqual(image.Title(), 'bar')
        self.assertEqual(image.Description(), 'baz')

    def test_imageCaption(self):
        imageCaption = self.n1.imageCaption
        self.assertIsNone(imageCaption())
        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=dummy_image(zptlogo))
        self.assertEqual(imageCaption(), 'baz')

    def test_image_thumb(self):
        thumb = self.n1.image_thumb

        # no images in news article, so no thumb must be present
        self.assertIsNone(thumb())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=dummy_image(zptlogo))
        self.assertIsNotNone(thumb())

    def test_tag(self):
        tag = self.n1.tag

        # no images in news article, so no tag must be present
        self.assertIsNone(tag())

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=dummy_image(zptlogo))

        # image title must be in both, alt and title attributes
        self.assertIn('alt="baz" title="bar"', tag())

        # image size
        self.assertIn('height="16" width="16', tag())

        # image scale using @@images
        self.assertIn('src="http://nohost/plone/test-folder/n1/foo/@@images/',
                      tag(scale='preview'))

        # image class
        self.assertIn('class="myClass"', tag(css_class='myClass'))



# -*- coding:utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone.app.upgrade.utils import loadMigrationProfile

import logging


def fix_collections(context):
    """Update to 1008 version."""
    logger = logging.getLogger(PROJECTNAME)
    collections = context.portal_catalog(portal_type='Collection')
    logger.info('About to update {0} objects'.format(len(collections)))
    for col in collections:
        obj = col.getObject()
        query = obj.getQuery()
        fixed_query = []
        for item in query:
            fixed_item = dict(item)
            if item['i'] == 'urgency':
                fixed_item['o'] = 'plone.app.querystring.operation.intselection.is'
            fixed_query.append(fixed_item)
        obj.setQuery(fixed_query)
        logger.info(
            'Collection %s at %s updated' % (col.id, col.getPath())
        )
    logger.info('Done')


def apply_profile(context):
    """Update to 1008 version."""
    logger = logging.getLogger(PROJECTNAME)
    profile = 'profile-collective.nitf.upgrades.v1008:default'
    loadMigrationProfile(context, profile)
    logger.info('Updated to version 1008')



# -*- coding: utf-8 -*-

from setuptools import find_packages
from setuptools import setup

import os

version = '1.0b6'
description = "A content type inspired on the IPTC's News Industry Text "
"Format specification."
long_description = (
    open("README.rst").read() + "\n" +
    open(os.path.join("docs", "CREDITS.rst")).read() + "\n" +
    open(os.path.join("docs", "CHANGES.rst")).read()
)

setup(name='collective.nitf',
      version=version,
      description=description,
      long_description=long_description,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Office/Business :: News/Diary",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone dexterity iptc newsml nitf',
      author='Héctor Velarde',
      author_email='hector.velarde@gmail.com',
      url='https://github.com/collective/collective.nitf',
      license='GPLv2',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'AccessControl',
        'Acquisition',
        'collective.js.galleria',
        'collective.js.jqueryui',
        'collective.prettydate>=1.1',
        'collective.z3cform.widgets>=1.0b9',
        'five.grok',
        'Pillow',
        'plone.api',
        'plone.app.dexterity[grok,relations]',
        'plone.app.imaging',
        'plone.app.lockingbehavior',
        'plone.app.portlets',
        'plone.app.querystring >=1.2.5',
        'plone.app.referenceablebehavior',
        'plone.app.registry',
        'plone.app.textfield',
        'plone.app.vocabularies',
        'plone.autoform',
        'plone.dexterity',
        'plone.directives.dexterity',
        'plone.directives.form',
        'plone.formwidget.contenttree',
        'plone.indexer',
        'plone.portlets',
        'plone.registry',
        'plone.uuid',
        'Products.CMFCore',
        'Products.CMFPlone>=4.2',
        'Products.GenericSetup',
        'setuptools',
        'z3c.form',
        'z3c.relationfield',
        'zope.browserpage',
        'zope.component',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.schema',
        ],
      extras_require={
        'test': [
          'plone.app.collection',
          'plone.app.customerize',
          'plone.app.robotframework',
          'plone.app.testing [robot] >=4.2.2',
          'plone.app.testing',
          'plone.browserlayer',
          'plone.namedfile[blobs]',
          'plone.testing',
          'robotsuite',
          'unittest2',
          ],
        },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from collective.nitf import _
from collective.nitf.content import INITF
from collective.nitf.controlpanel import INITFCharCountSettings
from collective.nitf.controlpanel import INITFSettings
from collective.nitf.interfaces import INITFLayer
from five import grok
from plone.directives import dexterity
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.utils import getToolByName
from warnings import warn
from zope.component import getUtility
from zope.interface import Interface
from plone.app.imaging.scaling import ImageScaling as BaseImageScaling

import json
import mimetypes


grok.templatedir('templates')


# TODO: enable_form_tabbing must be user selectable
class AddForm(dexterity.AddForm):
    """ Default view looks like a News Item.
    """
    grok.name('collective.nitf.content')
    grok.layer(INITFLayer)
    grok.context(INITF)
    schema = INITF

    def update(self):
        super(AddForm, self).update()
        # iterate over fieldsets
        for group in self.groups:
            # HACK: we need to update criteria of the ObjPathSourceBinder here
            # as we want to list only relatable content types
            if 'relatedItems' in group.widgets.keys():
                registry = getUtility(IRegistry)
                settings = registry.forInterface(INITFSettings)
                widget = group.widgets['relatedItems']
                criteria = widget.source.selectable_filter.criteria
                criteria['portal_type'] = settings.relatable_content_types

    def updateWidgets(self):
        super(AddForm, self).updateWidgets()
        # XXX why we need to do this?
        self.widgets['subtitle'].style = u'width: 100%;'
        self.widgets['IDublinCore.description'].rows = 3
        self.widgets['IDublinCore.description'].style = u'width: 100%;'


class EditForm(dexterity.EditForm):
    """ Default view looks like a News Item.
    """
    grok.context(INITF)
    grok.layer(INITFLayer)
    schema = INITF

    def update(self):
        super(EditForm, self).update()
        # iterate over fieldsets
        for group in self.groups:
            # HACK: we need to update criteria of the ObjPathSourceBinder here
            # as we want to list only relatable content types
            if 'relatedItems' in group.widgets.keys():
                registry = getUtility(IRegistry)
                settings = registry.forInterface(INITFSettings)
                widget = group.widgets['relatedItems']
                criteria = widget.source.selectable_filter.criteria
                criteria['portal_type'] = settings.relatable_content_types

    def updateWidgets(self):
        super(EditForm, self).updateWidgets()
        # XXX why we need to do this?
        self.widgets['subtitle'].style = u'width: 100%;'
        self.widgets['IDublinCore.description'].rows = 3
        self.widgets['IDublinCore.description'].style = u'width: 100%;'


class View(dexterity.DisplayForm):
    """ Default view looks like a News Item.
    """
    grok.context(INITF)
    grok.require('zope2.View')
    grok.layer(INITFLayer)

    def update(self):
        self.context = aq_inner(self.context)

    def _get_brains(self, object_name=None):
        """ Return a list of brains inside the NITF object.
        """
        catalog = getToolByName(self.context, 'portal_catalog')
        path = '/'.join(self.context.getPhysicalPath())
        brains = catalog(Type=object_name, path=path,
                         sort_on='getObjPositionInParent')

        return brains

    def get_images(self):
        """ Return a list of image brains inside the NITF object.
        """
        return self._get_brains('Image')

    def has_images(self):
        """ Return the number of images inside the NITF object.
        """
        return len(self.get_images())

    def get_files(self):
        """ Return a list of file brains inside the NITF object.
        """
        return self._get_brains('File')

    def has_files(self):
        """ Return the number of files inside the NITF object.
        """
        return len(self.get_files())

    def get_links(self):
        """ Return a list of link brains inside the NITF object.
        """
        return self._get_brains('Link')

    def has_links(self):
        """ Return the number of links inside the NITF object.
        """
        return len(self.get_links())

    def get_media(self):
        """ Return a list of object brains inside the NITF object.
        """
        media_ct = [x.title for x in self.context.allowedContentTypes()]
        return self._get_brains(media_ct)

    def has_media(self):
        """ Return the number of media inside the NITF object.
        """
        return len(self.get_media())

    def getText(self):
        warn("Calling getText is deprecated. Use self.text.output instead.",
             DeprecationWarning)
        return self.text.output

    # The purpose of these methods is to emulate those on News Item
    def getImage(self):
        warn("Calling getImage on the view is deprecated. Call it on the object.",
             DeprecationWarning)
        return self.context.getImage()

    def imageCaption(self):
        warn("Calling imageCaption on the view is deprecated. Call it on the object.",
             DeprecationWarning)
        return self.context.imageCaption()

    def tag(self, **kwargs):
        warn("Calling tag on the view is deprecated. Call it on the object.",
             DeprecationWarning)
        return self.context.tag(**kwargs)


# TODO: get rid of this class
class Display_Macros(View):
    grok.context(INITF)
    grok.require('zope2.View')
    grok.layer(INITFLayer)


class Folder_Summary_View(grok.View):
    grok.context(Interface)
    grok.layer(INITFLayer)
    grok.name("folder_summary_view")
    grok.require('zope2.View')


class NITF(View):
    """ Shows news article in NITF XML format.
    """
    grok.context(INITF)
    grok.layer(INITFLayer)
    grok.name('nitf')
    grok.require('zope2.View')

    def update(self):
        self.context = aq_inner(self.context)
        self.uuid = IUUID(self.context)

    def _get_mediatype(self, mimetype):
        """ Return one of the possible values of the media-type controlled
        vocabulary.
        """
        # 'data' and 'other' are also part of the controlled vocabulary; we
        # are not going to use 'data'
        vocabulary = ['text', 'audio', 'image', 'video', 'application']
        for i in vocabulary:
            if mimetype.find(i) != -1:
                return i

        return 'other'

    MEDIA = """
<media media-type="%s">
    <media-reference mime-type="%s" source="%s" alternate-text="%s"%s%s />
    <media-caption>%s</media-caption>
</media>"""

    def get_media(self):
        """ Return a list of object brains inside the NITF object.
        """
        media = []
        # XXX: we could honor original order calling the get_media() method in
        # View; how can we do that?
        results = self.get_images() + self.get_files()
        for r in results:
            obj = r.getObject()
            source = obj.absolute_url()
            (mimetype, encoding) = mimetypes.guess_type(source)
            # if no mime type is detected, result is None; we must change it
            mimetype = mimetype and mimetype or ''
            mediatype = self._get_mediatype(mimetype)
            alternate_text = obj.Title()
            caption = obj.Description()
            # we only include height and/or width if we have a value for them
            height = obj.getHeight()
            height = height and ' height="%s"' % obj.getHeight() or ''
            width = obj.getWidth()
            width = width and ' width="%s"' % obj.getWidth() or ''
            m = self.MEDIA % (mediatype, mimetype, source, alternate_text,
                              height, width, caption)
            media.append(m)

        return media


class NewsML(View):
    """ Shows news article in NewsML XML format.
    """
    grok.context(INITF)
    grok.layer(INITFLayer)
    grok.name('newsml')
    grok.require('zope2.View')

    def version(self):
        """ Returns news article revision number.
        """
        # TODO: get revision number
        return 1

    def nitf_size(self):
        """ Returns size of the news article in NITF format.
        """
        # TODO: calculate size
        return 1000

    ITEM_REF = """
<itemRef href="%s/@@nitf" size="%s"
   contenttype="application/nitf+xml" format="fmt:nitf">
    <title>%s</title>
</itemRef>"""

    def get_related_items(self):
        """ Returns an itemRef tag for each related item (only News Articles).
        """
        items = getattr(self.context, 'relatedItems', None)
        if items is not None:
            related_items = []
            for i in items:
                href = i.to_object.absolute_url()
                size = self.nitf_size()
                title = i.to_object.Title()
                item_ref = self.ITEM_REF % (href, size, title)
                related_items.append(item_ref)
            return related_items


class Media(View):
    grok.context(INITF)
    grok.require('cmf.ModifyPortalContent')


class DeleteMedia(View):
    grok.context(INITF)
    grok.name('delete_media')
    grok.require('cmf.ModifyPortalContent')

    # XXX: This is here, because under certain situations, grok will get the
    # template of 'View' superclass and raise an exception.
    template = None

    def __call__(self):
        delete_id = self.request['id'] if 'id' in self.request else None

        status = {'status': 'error'}
        if delete_id:
            self.context.manage_delObjects([delete_id])
            status['status'] = 'success'

        return json.dumps(status)

    def render(self):
        pass


class CharactersCount(grok.View):
    grok.context(Interface)
    grok.name('characters-count.js')
    grok.require('zope2.View')

    def render(self):

        response = self.request.response
        response.setHeader('content-type', 'text/javascript;;charset=utf-8')
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFCharCountSettings)

        count_title = settings.show_title_counter
        count_description = settings.show_description_counter

        counter_text = _(u'Characters left: ')

        config_title = {
            'allowed': settings.title_max_chars,
            'optimal': settings.title_optimal_chars,
            'counterText': counter_text
        }
        config_description = {
            'allowed': settings.description_max_chars,
            'optimal': settings.description_optimal_chars,
            'counterText': counter_text
        }

        title = ''
        description = ''

        if count_title:
            title = '$("#form-widgets-IDublinCore-title").charCount(%s);' % json.dumps(config_title)

        if count_description:
            description = '$("#form-widgets-IDublinCore-description").charCount(%s);' % json.dumps(config_description)

        script = '$(document).ready(function() {%s %s});' % (title, description)
        return script


class Nitf_Galleria(View):
    grok.context(INITF)
    grok.layer(INITFLayer)
    grok.require('zope2.View')
    grok.name('nitf_galleria')

    def imagesJson(self):
        """ """
        try:
            img_brains = self.get_images()
        except IndexError:
            img_brains = None
        if img_brains:
            data = [{'image': str(brain.getPath() + '/image_preview'),
                     'title': brain.Title,
                     'description': brain.Description, 'right': brain.Rights(),
                     'link': brain.getURL()} for brain in img_brains]
        else:
            data = []

        return json.dumps(data)


class ImageScaling(BaseImageScaling):
    """ view used for generating (and storing) image scales """

    def __init__(self, context, request):
        self.image = context.getImage()
        self.context = self.image or context
        self.request = request

    def scale(self, fieldname=None, scale=None, height=None, width=None,
              **parameters):
        if not self.image:
            return None
        view = self.image.restrictedTraverse('@@images')
        return view.scale(fieldname, scale, height, width, **parameters)



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from collective.nitf.controlpanel import INITFCharCountSettings
from collective.nitf.interfaces import INITFLayer
from collective.nitf.testing import INTEGRATION_TESTING
from collective.nitf.tests.test_content import zptlogo
from plone.app.customerize import registration
from plone.app.testing import TEST_USER_ID, logout, setRoles
from plone.registry.interfaces import IRegistry
from StringIO import StringIO
from zope.component import getMultiAdapter, queryMultiAdapter, getUtility
from zope.interface import directlyProvides

import unittest2 as unittest


class DefaultViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, INITFLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_default_view_is_registered(self):
        pt = self.portal['portal_types']
        self.assertEqual(pt['collective.nitf.content'].default_view, 'view')

        registered = [v.name for v in registration.getViews(INITFLayer)]
        self.assertIn('view', registered)

    def test_get_images(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(len(view.get_images()), 0)

        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        images = view.get_images()
        self.assertEqual(len(images), 1)
        self.assertEqual(images[0].getObject().id, 'foo')
        self.assertEqual(images[0].getObject().Title(), 'bar')
        self.assertEqual(images[0].getObject().Description(), 'baz')

    def test_has_images(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        has_images = view.has_images
        self.assertEqual(has_images(), 0)

        self.n1.invokeFactory('Image', 'foo', image=StringIO(zptlogo))
        self.assertEqual(has_images(), 1)

    def test_get_files(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(len(view.get_files()), 0)

        self.n1.invokeFactory('File', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        files = view.get_files()
        self.assertEqual(len(files), 1)
        self.assertEqual(files[0].getObject().id, 'foo')
        self.assertEqual(files[0].getObject().Title(), 'bar')
        self.assertEqual(files[0].getObject().Description(), 'baz')

    def test_has_files(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(view.has_files(), 0)

        self.n1.invokeFactory('File', 'foo', image=StringIO(zptlogo))
        self.assertEqual(view.has_files(), 1)

    def test_get_links(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(len(view.get_links()), 0)

        self.n1.invokeFactory('Link', 'foo', remoteUrl='http://foo/')
        links = view.get_links()
        self.assertEqual(len(links), 1)
        self.assertEqual(links[0].getObject().id, 'foo')
        self.assertEqual(links[0].getObject().remoteUrl, 'http://foo/')

    def test_has_links(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(view.has_links(), 0)

        self.n1.invokeFactory('Link', 'foo', remoteUrl='http://foo/')
        self.assertEqual(view.has_links(), 1)

    def test_get_media(self):
        view = getMultiAdapter((self.n1, self.request), name='view')
        self.assertEqual(len(view.get_media()), 0)

        self.n1.invokeFactory('Image', 'foo', image=StringIO(zptlogo))
        self.n1.invokeFactory('File', 'bar', file=StringIO(zptlogo))
        self.n1.invokeFactory('Link', 'baz', remoteUrl='http://baz/')
        media = view.get_media()
        self.assertEqual(len(media), 3)
        self.assertEqual(media[0].getObject().id, 'foo')
        self.assertEqual(media[1].getObject().id, 'bar')
        self.assertEqual(media[2].getObject().id, 'baz')


class NITFViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, INITFLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_get_mediatype(self):
        view = getMultiAdapter((self.n1, self.request), name='nitf')
        _get_mediatype = view._get_mediatype
        self.assertEqual(_get_mediatype('application/pdf'), 'application')
        self.assertEqual(_get_mediatype('audio/mpeg3'), 'audio')
        self.assertEqual(_get_mediatype('image/jpeg'), 'image')
        self.assertEqual(_get_mediatype('image/png'), 'image')
        self.assertEqual(_get_mediatype('multipart/signed'), 'other')
        self.assertEqual(_get_mediatype('text/plain'), 'text')
        self.assertEqual(_get_mediatype('video/avi'), 'video')

    def test_get_media(self):
        view = getMultiAdapter((self.n1, self.request), name='nitf')
        get_media = view.get_media
        self.assertEqual(len(get_media()), 0)

        self.n1.invokeFactory('Image', 'foo', image=StringIO(zptlogo))
        self.assertEqual(len(get_media()), 1)


class NewsMLViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, INITFLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_newsml_view_is_registered(self):
        view = queryMultiAdapter((self.n1, self.request), name='newsml')
        self.assertIsNotNone(view)


class GalleriaViewTestCase(unittest.TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, INITFLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_nitf_galleria_view_is_registered(self):
        registered = [v.name for v in registration.getViews(INITFLayer)]
        self.assertIn('nitf_galleria', registered)

        view = queryMultiAdapter((self.n1, self.request), name='nitf_galleria')
        self.assertIsNotNone(view)


class MediaViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, INITFLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_media_view_is_registered(self):
        view = queryMultiAdapter((self.n1, self.request), name='media')
        self.assertIsNotNone(view)

    def test_media_view_is_protected(self):
        logout()
        self.assertRaises(Unauthorized, self.n1.restrictedTraverse, '@@media')


class CharacterCountJSTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_config_empty(self):
        view = getMultiAdapter((self.folder, self.request), name='characters-count.js')
        render = view.render()
        self.assertEqual(render, '$(document).ready(function() { });')

    def test_config_nitf(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFCharCountSettings)
        settings.show_title_counter = True
        settings.show_description_counter = True
        view = getMultiAdapter((self.n1.restrictedTraverse('edit'), self.request), name='characters-count.js')
        render = view.render()
        self.assertEqual(
            render,
            '$(document).ready(function() {\
$("#form-widgets-IDublinCore-title").charCount({"optimal": 100, \
"counterText": "Characters left: ", "allowed": 100}); \
$("#form-widgets-IDublinCore-description").charCount({"optimal": 200, \
"counterText": "Characters left: ", "allowed": 200});});'
        )


class TraversalViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

        self.folder.invokeFactory('collective.nitf.content', 'n1')
        self.n1 = self.folder['n1']

    def test_images_traversal(self):
        self.n1.invokeFactory('Image', 'foo', title='bar', description='baz',
                              image=StringIO(zptlogo))
        view = getMultiAdapter((self.n1, self.request), name='images')

        image = view.scale('image')
        self.assertEqual(image.data, zptlogo)



# -*- coding: utf-8 -*-

from collective.nitf.controlpanel import INITFSettings
from collective.nitf.testing import INTEGRATION_TESTING
from plone.registry.interfaces import IRegistry
from zope.component import getUtility, queryUtility
from zope.schema.interfaces import IVocabularyFactory

import unittest2 as unittest


class VocabulariesTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        registry = getUtility(IRegistry)
        self.settings = registry.forInterface(INITFSettings)

    def test_genres_vocabulary(self):
        name = 'collective.nitf.Genres'
        util = queryUtility(IVocabularyFactory, name)
        self.assertTrue(util is not None)
        genres = util(self.portal)
        self.assertEqual(len(genres), 44)

    def test_available_genres_vocabulary(self):
        name = 'collective.nitf.AvailableGenres'
        util = queryUtility(IVocabularyFactory, name)
        self.assertTrue(util is not None)
        available_genres = util(self.portal)
        # FIXME: we need to set up at least one genre
        self.assertEqual(len(available_genres), 0)
        # self.assertTrue(u'Current' in available_genres)

    def test_available_genres_vocabulary_is_sorted(self):
        self.settings.available_genres = [u"Voicer", u"Current", u"Actuality"]
        name = 'collective.nitf.AvailableGenres'
        util = getUtility(IVocabularyFactory, name)
        genres = util(self.portal)
        genres = [i.title for i in genres]
        self.assertListEqual(genres, [u"Actuality", u"Current", u"Voicer"])

    def test_available_sections_vocabulary(self):
        name = 'collective.nitf.AvailableSections'
        util = queryUtility(IVocabularyFactory, name)
        self.assertTrue(util is not None)
        sections = util(self.portal)
        # FIXME: we need to set up at least one section
        self.assertEqual(len(sections), 0)
        # self.assertTrue(u'Default' in sections)

    def test_available_sections_vocabulary_is_sorted(self):
        self.settings.available_sections = set([u"5", u"4", u"3", u"2", u"1"])
        name = 'collective.nitf.AvailableSections'
        util = getUtility(IVocabularyFactory, name)
        sections = util(self.portal)
        sections = [i.title for i in sections]
        self.assertListEqual(sections, [u"1", u"2", u"3", u"4", u"5"])

    def test_urgencies_vocabulary(self):
        name = 'collective.nitf.Urgencies'
        util = queryUtility(IVocabularyFactory, name)
        self.assertTrue(util is not None)
        urgencies = util(self.portal)
        self.assertEqual(len(urgencies), 3)



# -*- coding:utf-8 -*-
from collective.nitf.config import PROJECTNAME
from plone.app.upgrade.utils import loadMigrationProfile

import logging


def apply_profile(context):
    """Update to 1007 version."""
    logger = logging.getLogger(PROJECTNAME)
    profile = 'profile-collective.nitf.upgrades.v1007:default'
    loadMigrationProfile(context, profile)
    logger.info('Updated to version 1007')



# -*- coding: utf-8 -*-

from collective.nitf.testing import ROBOT_TESTING
from plone.testing import layered

import os
import robotsuite
import unittest

dirname = os.path.dirname(__file__)
files = os.listdir(dirname)
tests = [f for f in files if f.startswith('test_') and f.endswith('.robot')]


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests([
        layered(
            robotsuite.RobotTestSuite(t, noncritical=['Expected Failure']),
            layer=ROBOT_TESTING)
        for t in tests
    ])
    return suite



# -*- coding: utf-8 -*-

from AccessControl.unauthorized import Unauthorized
from collective.nitf import _
from collective.prettydate.interfaces import IPrettyDate
from five import grok
from plone.app.portlets.portlets import base
from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from plone.portlets.interfaces import IPortletDataProvider
from Products.CMFCore.utils import getToolByName
from zope import schema
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter, getUtility
from zope.formlib import form
from zope.interface import implements
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary


class NitfFilterList(grok.View):
    grok.context(Interface)
    grok.name("nitf-filter-list")
    grok.require("zope2.View")

    template = ViewPageTemplateFile('nitf_filter_list.pt')

    def __call__(self):
        section = self.request.get('nitf-section-filter', None)
        collection_uid = self.request.get('nitf-filter-collection-uuid', None)
        if not section:
            section = None
        if not collection_uid:
            collection_uid = None

        result = self.render(section=section, uid=collection_uid)

        return result

    def getResults(self, section=None, limit=10, uid=None):
        uc = getToolByName(self.context, 'uid_catalog')
        pc = getToolByName(self.context, 'portal_catalog')

        collection = None
        query = {}
        if uid:
            try:
                collection = uc(UID=uid)[0].getObject()
                query = collection.buildQuery()
            except IndexError:
                pass

        # Let's force the portal_type to be collective.nitf.content
        query['Type'] = ('News Article',)

        # Now, let's try and load default values if not defined in the
        # collection
        if 'sort_on' not in query:
            query['sort_on'] = 'created'

            if 'sort_order' not in query:
                query['sort_order'] = 'reverse'

        if 'sort_limit' not in query:
            query['sort_limit'] = limit

        # Now, let's remove any trace of a section (this should be filtered
        # from portlet selection)
        if 'section' in query:
            del query['section']

        if section and section != 'all':
            util = getUtility(IVocabularyFactory,
                              name=u'collective.nitf.AvailableSections')
            vocab = util(self.context)

            query.update({'section': vocab.getTermByToken(section).value})

        results = pc(**query)[:limit]

        return results

    def render(self, pretty_date=True, **kw):
        results = self.getResults(**kw)
        return self.template(results=results, pretty_date=pretty_date)

    def getPrettyDate(self, date):
        # Returns human readable date for the tweet
        date_utility = getUtility(IPrettyDate)
        date = date_utility.date(date)

        return date


class ILatestSectionableNITFPortlet(IPortletDataProvider):
    """
    A portlet which shows the latest NITF created and can be filtered
    by section.
    """

    header = schema.TextLine(
        title=_(u'Header'),
        description=_(u"The header for the portlet. Leave empty for none."),
        required=False)

    limit = schema.Int(
        title=_(u"Limit"),
        description=_(u"Specify the maximum number of items to show in the "
                      u"portlet. Leave this blank to show all items."),
        default=10,
        required=False)

    pretty_date = schema.Bool(
        title=_(u'Pretty dates'),
        description=_(u"Show dates in a pretty format (ie. '4 hours ago')."),
        default=True,
        required=False)

    filter_collection = schema.Choice(
        title=_(u"Filter collection"),
        description=_(u"Use the criteria from this collection to modify the "
                      u"search results and order."),
        required=False,
        source=SearchableTextSourceBinder(
            {'portal_type': ('Topic', 'Collection')},
            default_query='path:'))


class Assignment(base.Assignment):
    """
    Portlet assignment.
    This is what is actually managed through the portlets UI and associated
    with columns.
    """

    implements(ILatestSectionableNITFPortlet)

    header = u""
    limit = 10
    pretty_date = True
    filter_collection = None

    def __init__(self,
                 header=u"",
                 limit=10,
                 pretty_date=True,
                 filter_collection=None):

        self.header = header
        self.limit = limit
        self.pretty_date = pretty_date
        self.filter_collection = filter_collection

    @property
    def title(self):
        """This property is used to give the title of the portlet in the
        "manage portlets" screen. Here, we use the title that the user gave.
        """
        return _(u"Latest Sectionable NITF")


class Renderer(base.Renderer):

    render = ViewPageTemplateFile('latest_sectionable_nitf.pt')

    def getHeader(self):
        """
        Returns the header for the portlet
        """
        return self.data.header

    def getResults(self, section=None):
        view = getMultiAdapter((self.context, self.request),
                               name="nitf-filter-list")

        uid = None
        if self.data.filter_collection:
            uid = self.getCollectionUID()

        results = view.render(section=section,
                              limit=self.data.limit,
                              pretty_date=self.data.pretty_date,
                              uid=uid)

        return results

    def getAvailableSections(self):
        vocab = getUtility(IVocabularyFactory,
                           name=u'collective.nitf.AvailableSections')(self.context)

        values = SimpleVocabulary([SimpleVocabulary.createTerm(_(u"All"),
                                                               "all",
                                                               _(u"All"))] +
                                  vocab._terms)

        return values

    def getCollectionUID(self):
        portal = self.context.portal_url.getPortalObject()
        context = portal

        for i in self.data.filter_collection.split('/')[1:]:
            try:
                context = context.restrictedTraverse(i)
            except Unauthorized:
                return ''

        if context is not portal and context.portal_type in ('Topic', 'Collection'):
            return context.UID()
        else:
            return ''


class AddForm(base.AddForm):

    form_fields = form.Fields(ILatestSectionableNITFPortlet)

    label = _(u"Add latest NITF Portlet")
    description = _(u"This portlet display a list of latest NITF created. "
                    u"It allows to filter NITF from different sections.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):

    form_fields = form.Fields(ILatestSectionableNITFPortlet)

    label = _(u"Add latest NITF Portlet")
    description = _(u"This portlet display a list of latest NITF created. "
                    u"It allows to filter NITF from different sections.")



# -*- coding: utf-8 -*-
from collective.nitf import _
from collective.nitf.config import GENRES
from collective.nitf.config import URGENCIES
from collective.nitf.controlpanel import INITFSettings
from five import grok
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary

import unicodedata


def _normalize_token(token):
    ''' Normalize a token using ascii as encoding '''
    normalize = unicodedata.normalize
    return normalize('NFKD', token).encode('ascii', 'ignore').lower()


class AvailableGenresVocabulary(object):
    """ Creates a vocabulary with the available genres stored in the registry;
    the vocabulary is normalized to allow the use of non-ASCII characters.
    """
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFSettings)
        available_genres = list(settings.available_genres)
        # sort by translated genre
        available_genres.sort(lambda a, b: cmp(_(a), _(b)))
        items = []
        for genre in available_genres:
            token = _normalize_token(genre)
            items.append(SimpleVocabulary.createTerm(genre, token, _(genre)))
        return SimpleVocabulary(items)

grok.global_utility(AvailableGenresVocabulary,
                    name=u'collective.nitf.AvailableGenres')


class SectionsVocabulary(object):
    """ Creates a vocabulary with the available sections stored in the
    registry; the vocabulary is normalized to allow the use of non-ASCII
    characters.
    """
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(INITFSettings)
        available_sections = list(settings.available_sections)
        available_sections.sort()
        items = []
        for section in available_sections:
            token = _normalize_token(section)
            items.append(SimpleVocabulary.createTerm(section, token, section))
        return SimpleVocabulary(items)

grok.global_utility(SectionsVocabulary,
                    name=u'collective.nitf.AvailableSections')


class GenresVocabulary(object):
    """ Creates a vocabulary to expose Genres
    """
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        return GENRES

grok.global_utility(GenresVocabulary, name=u'collective.nitf.Genres')


class UrgenciesVocabulary(object):
    """ Creates a vocabulary to expose Urgencies
    """
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        return URGENCIES

grok.global_utility(UrgenciesVocabulary, name=u'collective.nitf.Urgencies')



# -*- coding: utf-8 -*-

from collective.nitf.config import PROJECTNAME
from collective.nitf.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.browserlayer.utils import registered_layers

import unittest2 as unittest


class InstallTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_installed(self):
        qi = getattr(self.portal, 'portal_quickinstaller')
        self.assertTrue(qi.isProductInstalled(PROJECTNAME))

    def test_add_permission(self):
        permission = 'collective.nitf: Add News Article'
        roles = self.portal.rolesOfPermission(permission)
        roles = [r['name'] for r in roles if r['selected']]
        expected = ['Contributor', 'Manager', 'Owner', 'Site Administrator']
        self.assertListEqual(roles, expected)

    def test_addon_layer(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertTrue('INITFLayer' in layers,
                        'add-on layer was not installed')

    def test_link_workflow_changed(self):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Link', 'obj')
        obj = self.portal['obj']
        workflow_tool = self.portal.portal_workflow
        chain = workflow_tool.getChainForPortalType(obj.portal_type)
        self.assertEqual(len(chain), 1)
        self.assertEqual(chain[0], 'one_state_workflow',
                         'workflow was not changed for Link content type')

    def test_javascript_registry(self):
        portal_javascripts = self.portal.portal_javascripts
        resources = portal_javascripts.getResourceIds()
        # moved to sc.collapsible.edit
        self.assertFalse('++resource++collective.nitf/jquery.collapsible-v.2.1.3.js' in resources)
        self.assertTrue('++resource++collective.nitf/nitf_fixes.js' in resources)

    def test_upgrade_javascript_registry(self):
        portal_javascripts = self.portal.portal_javascripts
        resources = portal_javascripts.getResourceIds()
        self.assertTrue('++resource++collective.galleria.js' in resources)
        qi = self.portal.portal_quickinstaller
        setup = self.portal.portal_setup
        portal_javascripts.manage_removeScript('++resource++collective.galleria.js')
        resources = portal_javascripts.getResourceIds()
        self.assertFalse('++resource++collective.galleria.js' in resources)
        setup.setLastVersionForProfile(u'collective.nitf:default', '1001')
        qi.upgradeProduct('collective.nitf')
        resources = portal_javascripts.getResourceIds()
        self.assertTrue('++resource++collective.galleria.js' in resources)

    def test_css_registry(self):
        portal_css = self.portal.portal_css
        resources = portal_css.getResourceIds()
        # moved to sc.collapsible.edit
        self.assertFalse('++resource++collective.nitf/collapsible.css' in resources)


class UninstallTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.qi = getattr(self.portal, 'portal_quickinstaller')
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer_removed(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertFalse('INITFLayer' in layers,
                         'add-on layer was not removed')

    def test_javascript_registry_removed(self):
        portal_javascripts = self.portal.portal_javascripts
        resources = portal_javascripts.getResourceIds()
        self.assertFalse('++resource++collective.nitf/jquery.collapsible-v.2.1.3.js' in resources)
        self.assertFalse('++resource++collective.nitf/nitf_fixes.js' in resources)

    def test_css_registry_removed(self):
        portal_css = self.portal.portal_css
        resources = portal_css.getResourceIds()
        self.assertFalse('++resource++collective.nitf/collapsible.css' in resources)
